@@ -779,7 +779,7 @@ class ReferenceField(CharField):
     def get(self, record):
         if (record.value.get(self.name)
                 and record.value[self.name][0]
-                and record.value[self.name][1] >= 0):
+                and record.value[self.name][1] >= -1):
             return ','.join(map(str, record.value[self.name]))
         return None
 
@@ -791,6 +791,11 @@ class ReferenceField(CharField):
             if isinstance(ref_id, (tuple, list)):
                 ref_id, rec_name = ref_id
             else:
+                if ref_id:
+                    try:
+                        ref_id = int(ref_id)
+                    except ValueError:
+                        pass
                 if '%s,%s' % (ref_model, ref_id) == self.get(record):
                     rec_name = record.value.get(self.name + '.rec_name', '')
                 else:
